@@ -1,13 +1,11 @@
 /*****************************************************************************
- *  $Id: strerror.c 890 2011-01-20 01:54:21Z chris.m.dunlap $
- *****************************************************************************
  *  Written by Chris Dunlap <cdunlap@llnl.gov>.
- *  Copyright (C) 2007-2011 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2013 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  UCRL-CODE-155910.
  *
  *  This file is part of the MUNGE Uid 'N' Gid Emporium (MUNGE).
- *  For details, see <http://munge.googlecode.com/>.
+ *  For details, see <https://munge.googlecode.com/>.
  *
  *  MUNGE is free software: you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
@@ -52,21 +50,21 @@ munge_strerror (munge_err_t errnum)
         case EMUNGE_NO_MEMORY:
             return ("Out of memory");
         case EMUNGE_SOCKET:
-            return ("Munged communication error");
+            return ("Socket communication error");
         case EMUNGE_TIMEOUT:
-            return ("Munged timed-out");
+            return ("Socket timeout");
         case EMUNGE_BAD_CRED:
             return ("Invalid credential format");
         case EMUNGE_BAD_VERSION:
-            return ("Unrecognized credential version");
+            return ("Invalid credential version");
         case EMUNGE_BAD_CIPHER:
-            return ("Unrecognized credential cipher type");
+            return ("Invalid cipher type");
         case EMUNGE_BAD_MAC:
-            return ("Unrecognized credential mac type");
+            return ("Invalid MAC type");
         case EMUNGE_BAD_ZIP:
-            return ("Unrecognized credential zip type");
+            return ("Invalid compression type");
         case EMUNGE_BAD_REALM:
-            return ("Unrecognized credential security realm");
+            return ("Unrecognized security realm");
         case EMUNGE_CRED_INVALID:
             return ("Invalid credential");
         case EMUNGE_CRED_EXPIRED:
