@@ -1,13 +1,11 @@
 /*****************************************************************************
- *  $Id: munge.c 890 2011-01-20 01:54:21Z chris.m.dunlap $
- *****************************************************************************
  *  Written by Chris Dunlap <cdunlap@llnl.gov>.
- *  Copyright (C) 2007-2011 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2013 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  UCRL-CODE-155910.
  *
  *  This file is part of the MUNGE Uid 'N' Gid Emporium (MUNGE).
- *  For details, see <http://munge.googlecode.com/>.
+ *  For details, see <https://munge.googlecode.com/>.
  *
  *  MUNGE is free software: you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
@@ -54,7 +52,7 @@
  *  Command-Line Options
  *****************************************************************************/
 
-const char * const short_opts = ":hLVns:i:o:c:Cm:Mz:Zu:g:t:S:";
+const char * const short_opts = ":hLVns:i:o:c:Cm:Mz:Zu:U:g:G:t:S:";
 
 #include <getopt.h>
 struct option long_opts[] = {
@@ -72,7 +70,9 @@ struct option long_opts[] = {
     { "zip",          required_argument, NULL, 'z' },
     { "list-zips",    no_argument,       NULL, 'Z' },
     { "restrict-uid", required_argument, NULL, 'u' },
+    { "uid",          required_argument, NULL, 'U' },
     { "restrict-gid", required_argument, NULL, 'g' },
+    { "gid",          required_argument, NULL, 'G' },
     { "ttl",          required_argument, NULL, 't' },
     { "socket",       required_argument, NULL, 'S' },
     {  NULL,          0,                 NULL,  0  }
@@ -86,6 +86,8 @@ struct option long_opts[] = {
 struct conf {
     munge_ctx_t  ctx;                   /* munge context                     */
     munge_err_t  status;                /* error status munging the cred     */
+    uid_t        cuid;                  /* credential UID                    */
+    gid_t        cgid;                  /* credential GID                    */
     char        *string;                /* input from string instead of file */
     char        *fn_in;                 /* input filename, '-' for stdin     */
     char        *fn_out;                /* output filename, '-' for stdout   */
@@ -109,7 +111,10 @@ void   destroy_conf (conf_t conf);
 void   parse_cmdline (conf_t conf, int argc, char **argv);
 void   display_help (char *prog);
 void   display_strings (const char *header, munge_enum_t type);
+uid_t  query_uid (const char *user);
+gid_t  query_gid (const char *group);
 void   open_files (conf_t conf);
+int    encode_cred (conf_t conf);
 void   display_cred (conf_t conf);
 
 
@@ -125,7 +130,7 @@ main (int argc, char *argv[])
     const char  *p;
 
     if (posignal (SIGPIPE, SIG_IGN) == SIG_ERR) {
-        log_err (EMUNGE_SNAFU, LOG_ERR, "Unable to ignore signal=%d", SIGPIPE);
+        log_err (EMUNGE_SNAFU, LOG_ERR, "Failed to ignore signal=%d", SIGPIPE);
     }
     log_open_file (stderr, argv[0], LOG_INFO, LOG_OPT_PRIORITY);
     conf = create_conf ();
@@ -140,16 +145,13 @@ main (int argc, char *argv[])
     }
     if (rc < 0) {
         if (errno == ENOMEM) {
-            log_errno (EMUNGE_NO_MEMORY, LOG_ERR, "Unable to read input");
+            log_errno (EMUNGE_NO_MEMORY, LOG_ERR, "Failed to read input");
         }
         else {
             log_err (EMUNGE_SNAFU, LOG_ERR, "Read error");
         }
     }
-    conf->status = munge_encode (&conf->cred, conf->ctx,
-        conf->data, conf->dlen);
-
-    if (conf->status != EMUNGE_SUCCESS) {
+    if (encode_cred (conf) < 0) {
         if (!(p = munge_ctx_strerror (conf->ctx))) {
             p = munge_strerror (conf->status);
         }
@@ -170,12 +172,14 @@ create_conf (void)
     conf_t conf;
 
     if (!(conf = malloc (sizeof (struct conf)))) {
-        log_errno (EMUNGE_NO_MEMORY, LOG_ERR, "Unable to create conf");
+        log_errno (EMUNGE_NO_MEMORY, LOG_ERR, "Failed to allocate conf");
     }
     if (!(conf->ctx = munge_ctx_create ())) {
-        log_errno (EMUNGE_NO_MEMORY, LOG_ERR, "Unable to create conf ctx");
+        log_errno (EMUNGE_NO_MEMORY, LOG_ERR, "Failed to create conf ctx");
     }
     conf->status = -1;
+    conf->cuid = geteuid ();
+    conf->cgid = getegid ();
     conf->string = NULL;
     conf->fn_in = "-";
     conf->fn_out = "-";
@@ -197,13 +201,13 @@ destroy_conf (conf_t conf)
      */
     if (conf->fp_in != NULL) {
         if (fclose (conf->fp_in) < 0) {
-            log_errno (EMUNGE_SNAFU, LOG_ERR, "Unable to close infile");
+            log_errno (EMUNGE_SNAFU, LOG_ERR, "Failed to close input file");
         }
         conf->fp_in = NULL;
     }
     if (conf->fp_out != NULL) {
         if ((fclose (conf->fp_out) < 0) && (errno != EPIPE)) {
-            log_errno (EMUNGE_SNAFU, LOG_ERR, "Unable to close outfile");
+            log_errno (EMUNGE_SNAFU, LOG_ERR, "Failed to close output file");
         }
         conf->fp_out = NULL;
     }
@@ -226,14 +230,12 @@ destroy_conf (conf_t conf)
 void
 parse_cmdline (conf_t conf, int argc, char **argv)
 {
-    char          *prog;
-    int            c;
-    char          *p;
-    munge_err_t    e;
-    int            i;
-    long int       l;
-    struct passwd *pw_ptr;
-    struct group  *gr_ptr;
+    char        *prog;
+    int          c;
+    char        *p;
+    munge_err_t  e;
+    int          i;
+    long int     l;
 
     opterr = 0;                         /* suppress default getopt err msgs */
 
@@ -283,7 +285,7 @@ parse_cmdline (conf_t conf, int argc, char **argv)
                 e = munge_ctx_set (conf->ctx, MUNGE_OPT_CIPHER_TYPE, i);
                 if (e != EMUNGE_SUCCESS) {
                     log_err (EMUNGE_SNAFU, LOG_ERR,
-                        "Unable to set cipher type: %s",
+                        "Failed to set cipher type: %s",
                         munge_ctx_strerror (conf->ctx));
                 }
                 break;
@@ -295,12 +297,12 @@ parse_cmdline (conf_t conf, int argc, char **argv)
                 i = munge_enum_str_to_int (MUNGE_ENUM_MAC, optarg);
                 if ((i < 0) || !munge_enum_is_valid (MUNGE_ENUM_MAC, i)) {
                     log_err (EMUNGE_SNAFU, LOG_ERR,
-                        "Invalid mac type \"%s\"", optarg);
+                        "Invalid MAC type \"%s\"", optarg);
                 }
                 e = munge_ctx_set (conf->ctx, MUNGE_OPT_MAC_TYPE, i);
                 if (e != EMUNGE_SUCCESS) {
                     log_err (EMUNGE_SNAFU, LOG_ERR,
-                        "Unable to set mac type: %s",
+                        "Failed to set MAC type: %s",
                         munge_ctx_strerror (conf->ctx));
                 }
                 break;
@@ -317,7 +319,7 @@ parse_cmdline (conf_t conf, int argc, char **argv)
                 e = munge_ctx_set (conf->ctx, MUNGE_OPT_ZIP_TYPE, i);
                 if (e != EMUNGE_SUCCESS) {
                     log_err (EMUNGE_SNAFU, LOG_ERR,
-                        "Unable to set compression type: %s",
+                        "Failed to set compression type: %s",
                         munge_ctx_strerror (conf->ctx));
                 }
                 break;
@@ -326,51 +328,47 @@ parse_cmdline (conf_t conf, int argc, char **argv)
                 exit (EMUNGE_SUCCESS);
                 break;
             case 'u':
-                if ((pw_ptr = getpwnam (optarg)) != NULL) {
-                    i = pw_ptr->pw_uid;
-                }
-                else {
-                    errno = 0;
-                    l = strtol (optarg, &p, 10);
-                    if (((errno == ERANGE)
-                                && ((l == LONG_MIN) || (l == LONG_MAX)))
-                            || (optarg == p) || (*p != '\0')
-                            || (l < 0) || (l > INT_MAX)) {
-                        log_err (EMUNGE_SNAFU, LOG_ERR,
+                i = query_uid (optarg);
+                if (i < 0) {
+                    log_err (EMUNGE_SNAFU, LOG_ERR,
                             "Unrecognized user \"%s\"", optarg);
-                    }
-                    i = (int) l;
                 }
                 e = munge_ctx_set (conf->ctx, MUNGE_OPT_UID_RESTRICTION, i);
                 if (e != EMUNGE_SUCCESS) {
                     log_err (EMUNGE_SNAFU, LOG_ERR,
-                        "Unable to set uid restriction: %s",
+                        "Failed to set UID restriction: %s",
                         munge_ctx_strerror (conf->ctx));
                 }
                 break;
-            case 'g':
-                if ((gr_ptr = getgrnam (optarg)) != NULL) {
-                    i = gr_ptr->gr_gid;
+            case 'U':
+                i = query_uid (optarg);
+                if (i < 0) {
+                    log_err (EMUNGE_SNAFU, LOG_ERR,
+                            "Unrecognized user \"%s\"", optarg);
                 }
-                else {
-                    errno = 0;
-                    l = strtol (optarg, &p, 10);
-                    if (((errno == ERANGE)
-                                && ((l == LONG_MIN) || (l == LONG_MAX)))
-                            || (optarg == p) || (*p != '\0')
-                            || (l < 0) || (l > INT_MAX)) {
-                        log_err (EMUNGE_SNAFU, LOG_ERR,
+                conf->cuid = i;
+                break;
+            case 'g':
+                i = query_gid (optarg);
+                if (i < 0) {
+                    log_err (EMUNGE_SNAFU, LOG_ERR,
                             "Unrecognized group \"%s\"", optarg);
-                    }
-                    i = (int) l;
                 }
                 e = munge_ctx_set (conf->ctx, MUNGE_OPT_GID_RESTRICTION, i);
                 if (e != EMUNGE_SUCCESS) {
                     log_err (EMUNGE_SNAFU, LOG_ERR,
-                        "Unable to set gid restriction: %s",
+                        "Failed to set GID restriction: %s",
                         munge_ctx_strerror (conf->ctx));
                 }
                 break;
+            case 'G':
+                i = query_gid (optarg);
+                if (i < 0) {
+                    log_err (EMUNGE_SNAFU, LOG_ERR,
+                            "Unrecognized group \"%s\"", optarg);
+                }
+                conf->cgid = i;
+                break;
             case 't':
                 errno = 0;
                 l = strtol (optarg, &p, 10);
@@ -389,7 +387,7 @@ parse_cmdline (conf_t conf, int argc, char **argv)
                 e = munge_ctx_set (conf->ctx, MUNGE_OPT_TTL, (int) l);
                 if (e != EMUNGE_SUCCESS) {
                     log_err (EMUNGE_SNAFU, LOG_ERR,
-                        "Unable to set time-to-live: %s",
+                        "Failed to set time-to-live: %s",
                         munge_ctx_strerror (conf->ctx));
                 }
                 break;
@@ -397,7 +395,7 @@ parse_cmdline (conf_t conf, int argc, char **argv)
                 e = munge_ctx_set (conf->ctx, MUNGE_OPT_SOCKET, optarg);
                 if (e != EMUNGE_SUCCESS) {
                     log_err (EMUNGE_SNAFU, LOG_ERR,
-                        "Unable to set munge socket name: %s",
+                        "Failed to set munge socket name: %s",
                         munge_ctx_strerror (conf->ctx));
                 }
                 break;
@@ -412,7 +410,7 @@ parse_cmdline (conf_t conf, int argc, char **argv)
                 }
                 else {
                     log_err (EMUNGE_SNAFU, LOG_ERR,
-                        "Unable to process command-line");
+                        "Failed to process command-line");
                 }
                 break;
             case ':':
@@ -428,7 +426,7 @@ parse_cmdline (conf_t conf, int argc, char **argv)
                 }
                 else {
                     log_err (EMUNGE_SNAFU, LOG_ERR,
-                        "Unable to process command-line");
+                        "Failed to process command-line");
                 }
                 break;
             default:
@@ -512,14 +510,20 @@ display_help (char *prog)
     printf ("  %*s %s\n", w, "-u, --restrict-uid=UID",
             "Restrict credential decoding by UID");
 
+    printf ("  %*s %s\n", w, "-U, --uid=UID",
+            "Specify credential UID (requires root/CAP_SETUID)");
+
     printf ("  %*s %s\n", w, "-g, --restrict-gid=GID",
             "Restrict credential decoding by GID");
 
+    printf ("  %*s %s\n", w, "-G, --gid=GID",
+            "Specify credential GID (requires root/CAP_SETGID)");
+
     printf ("  %*s %s\n", w, "-t, --ttl=INTEGER",
             "Specify time-to-live (in seconds; 0=dfl -1=max)");
 
     printf ("  %*s %s\n", w, "-S, --socket=STRING",
-            "Specify local domain socket for daemon");
+            "Specify local domain socket for munged");
 
     printf ("\n");
     printf ("By default, payload read from stdin, "
@@ -547,6 +551,78 @@ display_strings (const char *header, munge_enum_t type)
 }
 
 
+uid_t
+query_uid (const char *user)
+{
+/*  Returns the uid for [user], or -1 on error.
+ */
+    struct passwd *pw_ptr;
+    uid_t          uid;
+    long int       luid;
+    char          *end_ptr;
+
+    if (user == NULL) {
+        errno = EINVAL;
+        return (-1);
+    }
+    pw_ptr = getpwnam (user);
+    if (pw_ptr != NULL) {
+        uid = pw_ptr->pw_uid;
+    }
+    else {
+        errno = 0;
+        luid = strtol (user, &end_ptr, 10);
+        if ((errno == ERANGE) && ((luid == LONG_MIN) || (luid == LONG_MAX))) {
+            return (-1);
+        }
+        if ((user == end_ptr) || (*end_ptr != '\0')) {
+            return (-1);
+        }
+        if ((luid < 0) || (luid > INT_MAX)) {
+            return (-1);
+        }
+        uid = (int) luid;
+    }
+    return (uid);
+}
+
+
+gid_t
+query_gid (const char *group)
+{
+/*  Returns the gid for [group], or -1 on error.
+ */
+    struct group  *gr_ptr;
+    gid_t          gid;
+    long int       lgid;
+    char          *end_ptr;
+
+    if (group == NULL) {
+        errno = EINVAL;
+        return (-1);
+    }
+    gr_ptr = getgrnam (group);
+    if (gr_ptr != NULL) {
+        gid = gr_ptr->gr_gid;
+    }
+    else {
+        errno = 0;
+        lgid = strtol (group, &end_ptr, 10);
+        if ((errno == ERANGE) && ((lgid == LONG_MIN) || (lgid == LONG_MAX))) {
+            return (-1);
+        }
+        if ((group == end_ptr) || (*end_ptr != '\0')) {
+            return (-1);
+        }
+        if ((lgid < 0) || (lgid > INT_MAX)) {
+            return (-1);
+        }
+        gid = (int) lgid;
+    }
+    return (gid);
+}
+
+
 void
 open_files (conf_t conf)
 {
@@ -556,7 +632,7 @@ open_files (conf_t conf)
         }
         else if (!(conf->fp_in = fopen (conf->fn_in, "r"))) {
             log_errno (EMUNGE_SNAFU, LOG_ERR,
-                "Unable to read from \"%s\"", conf->fn_in);
+                "Failed to read from \"%s\"", conf->fn_in);
         }
     }
     if (conf->fn_out) {
@@ -565,13 +641,56 @@ open_files (conf_t conf)
         }
         else if (!(conf->fp_out = fopen (conf->fn_out, "w"))) {
             log_errno (EMUNGE_SNAFU, LOG_ERR,
-                "Unable to write to \"%s\"", conf->fn_out);
+                "Failed to write to \"%s\"", conf->fn_out);
         }
     }
     return;
 }
 
 
+int
+encode_cred (conf_t conf)
+{
+/*  Encodes the credential based on the configuration [conf].
+ *  Returns 0 on success, -1 on error.
+ */
+    uid_t euid;
+    gid_t egid;
+
+    euid = geteuid ();
+    egid = getegid ();
+
+    if (egid != conf->cgid) {
+        if (setegid (conf->cgid) < 0) {
+            log_errno (errno, LOG_ERR,
+                    "Failed to create credential for GID %u", conf->cgid);
+        }
+    }
+    if (euid != conf->cuid) {
+        if (seteuid (conf->cuid) < 0) {
+            log_errno (errno, LOG_ERR,
+                    "Failed to create credential for UID %u", conf->cuid);
+        }
+    }
+    conf->status = munge_encode (&conf->cred, conf->ctx,
+            conf->data, conf->dlen);
+
+    if (euid != conf->cuid) {
+        if (seteuid (euid) < 0) {
+            log_errno (errno, LOG_ERR,
+                    "Failed to restore privileges for UID %u", euid);
+        }
+    }
+    if (egid != conf->cgid) {
+        if (setegid (egid) < 0) {
+            log_errno (errno, LOG_ERR,
+                    "Failed to restore privileges for GID %u", egid);
+        }
+    }
+    return ((conf->status == EMUNGE_SUCCESS) ? 0 : -1);
+}
+
+
 void
 display_cred (conf_t conf)
 {
