@@ -1,13 +1,11 @@
 /*****************************************************************************
- *  $Id: cipher.c 890 2011-01-20 01:54:21Z chris.m.dunlap $
- *****************************************************************************
  *  Written by Chris Dunlap <cdunlap@llnl.gov>.
- *  Copyright (C) 2007-2011 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2013 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  UCRL-CODE-155910.
  *
  *  This file is part of the MUNGE Uid 'N' Gid Emporium (MUNGE).
- *  For details, see <http://munge.googlecode.com/>.
+ *  For details, see <https://munge.googlecode.com/>.
  *
  *  MUNGE is free software: you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
@@ -447,7 +445,7 @@ _cipher_final (cipher_ctx *x, void *dst, int *dstlen)
         pad = x->buf[x->blklen - 1];
         if ((pad <= 0) || (pad > x->blklen)) {
             log_msg (LOG_DEBUG,
-                "Final decryption block has invalid pad=%d", pad);
+                "Final decryption block has invalid pad of %d", pad);
             return (-1);
         }
         for (i = x->blklen - pad; i < x->blklen; i++) {
