@@ -1,13 +1,11 @@
 /*****************************************************************************
- *  $Id: conf.c 890 2011-01-20 01:54:21Z chris.m.dunlap $
- *****************************************************************************
  *  Written by Chris Dunlap <cdunlap@llnl.gov>.
- *  Copyright (C) 2007-2011 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2013 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  UCRL-CODE-155910.
  *
  *  This file is part of the MUNGE Uid 'N' Gid Emporium (MUNGE).
- *  For details, see <http://munge.googlecode.com/>.
+ *  For details, see <https://munge.googlecode.com/>.
  *
  *  MUNGE is free software: you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
@@ -61,26 +59,40 @@
  *  Command-Line Options
  *****************************************************************************/
 
-const char * const short_opts = ":hLVfFS:";
+#define OPT_ADVICE              256
+#define OPT_KEY_FILE            257
+#define OPT_NUM_THREADS         258
+#define OPT_AUTH_SERVER         259
+#define OPT_AUTH_CLIENT         260
+#define OPT_BENCHMARK           264
+#define OPT_GROUP_CHECK         261
+#define OPT_GROUP_UPDATE        262
+#define OPT_SYSLOG              263
+#define OPT_LAST                265
+
+const char * const short_opts = ":hLVfFMS:";
 
 #include <getopt.h>
 struct option long_opts[] = {
-    { "help",              no_argument,       NULL, 'h' },
-    { "license",           no_argument,       NULL, 'L' },
-    { "version",           no_argument,       NULL, 'V' },
-    { "force",             no_argument,       NULL, 'f' },
-    { "foreground",        no_argument,       NULL, 'F' },
-    { "socket",            required_argument, NULL, 'S' },
-    { "advice",            no_argument,       NULL, 'A' },
-    { "key-file",          required_argument, NULL, '0' },
-    { "num-threads",       required_argument, NULL, '1' },
+    { "help",              no_argument,       NULL, 'h'              },
+    { "license",           no_argument,       NULL, 'L'              },
+    { "version",           no_argument,       NULL, 'V'              },
+    { "force",             no_argument,       NULL, 'f'              },
+    { "foreground",        no_argument,       NULL, 'F'              },
+    { "mlockall",          no_argument,       NULL, 'M'              },
+    { "socket",            required_argument, NULL, 'S'              },
+    { "advice",            no_argument,       NULL, OPT_ADVICE       },
+    { "key-file",          required_argument, NULL, OPT_KEY_FILE     },
+    { "num-threads",       required_argument, NULL, OPT_NUM_THREADS  },
 #if defined(AUTH_METHOD_RECVFD_MKFIFO) || defined(AUTH_METHOD_RECVFD_MKNOD)
-    { "auth-server-dir",   required_argument, NULL, '2' },
-    { "auth-client-dir",   required_argument, NULL, '3' },
+    { "auth-server-dir",   required_argument, NULL, OPT_AUTH_SERVER  },
+    { "auth-client-dir",   required_argument, NULL, OPT_AUTH_CLIENT  },
 #endif /* AUTH_METHOD_RECVFD_MKFIFO || AUTH_METHOD_RECVFD_MKNOD */
-    { "group-check-mtime", required_argument, NULL, '4' },
-    { "group-update-time", required_argument, NULL, '5' },
-    {  NULL,               0,                 NULL,  0  }
+    { "benchmark",         no_argument,       NULL, OPT_BENCHMARK    },
+    { "group-check-mtime", required_argument, NULL, OPT_GROUP_CHECK  },
+    { "group-update-time", required_argument, NULL, OPT_GROUP_UPDATE },
+    { "syslog",            no_argument,       NULL, OPT_SYSLOG       },
+    {  NULL,               0,                 NULL,  0               }
 };
 
 
@@ -108,15 +120,18 @@ create_conf (void)
     conf_t conf;
 
     if (!(conf = malloc (sizeof (struct conf)))) {
-        log_errno (EMUNGE_NO_MEMORY, LOG_ERR, "Unable to create conf");
+        log_errno (EMUNGE_NO_MEMORY, LOG_ERR, "Failed to allocate conf");
     }
     conf->ld = -1;
+    conf->got_benchmark = 0;
     conf->got_clock_skew = 1;
     conf->got_force = 0;
     conf->got_foreground = 0;
     conf->got_group_stat = !! MUNGE_GROUP_STAT_FLAG;
-    conf->got_socket_retry = !! MUNGE_SOCKET_RETRY_FLAG;
+    conf->got_mlockall = 0;
     conf->got_root_auth = !! MUNGE_AUTH_ROOT_ALLOW_FLAG;
+    conf->got_socket_retry = !! MUNGE_SOCKET_RETRY_FLAG;
+    conf->got_syslog = 0;
     conf->def_cipher = MUNGE_DEFAULT_CIPHER;
     conf->def_zip = zip_select_default_type (MUNGE_DEFAULT_ZIP);
     conf->def_mac = MUNGE_DEFAULT_MAC;
@@ -126,45 +141,52 @@ create_conf (void)
      *  FIXME: Add support for default realm.
      */
     conf->config_name = NULL;
+    conf->lockfile_fd = -1;
+    conf->lockfile_name = NULL;
+
     if (!(conf->logfile_name = strdup (MUNGED_LOGFILE))) {
         log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
-            "Cannot dup logfile name string");
+            "Failed to copy logfile name string");
     }
     if (!(conf->pidfile_name = strdup (MUNGED_PIDFILE))) {
         log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
-            "Cannot dup pidfile name string");
+            "Failed to copy pidfile name string");
     }
     if (!(conf->socket_name = strdup (MUNGE_SOCKET_NAME))) {
-        log_errno (EMUNGE_NO_MEMORY, LOG_ERR, "Cannot dup socket name string");
+        log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
+            "Failed to copy socket name string");
     }
     if (!(conf->seed_name = strdup (MUNGED_RANDOM_SEED))) {
-        log_errno (EMUNGE_NO_MEMORY, LOG_ERR, "Cannot dup seed name string");
+        log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
+            "Failed to copy seed name string");
     }
     if (!(conf->key_name = strdup (MUNGED_SECRET_KEY))) {
-        log_errno (EMUNGE_NO_MEMORY, LOG_ERR, "Cannot dup key name string");
+        log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
+            "Failed to copy key name string");
     }
     conf->dek_key = NULL;
     conf->dek_key_len = 0;
     conf->mac_key = NULL;
     conf->mac_key_len = 0;
+    memset (&conf->addr, 0, sizeof (conf->addr));
     conf->gids = NULL;
     conf->gids_update_secs = MUNGE_GROUP_UPDATE_SECS;
     conf->nthreads = MUNGE_THREADS;
+    conf->auth_server_dir = NULL;
+    conf->auth_client_dir = NULL;
+    conf->auth_rnd_bytes = MUNGE_AUTH_RND_BYTES;
 
 #if defined(AUTH_METHOD_RECVFD_MKFIFO) || defined(AUTH_METHOD_RECVFD_MKNOD)
-
     if (!(conf->auth_server_dir = strdup (MUNGE_AUTH_SERVER_DIR))) {
         log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
-            "Cannot dup auth-server-dir default string");
+            "Failed to copy auth-server-dir default string");
     }
     if (!(conf->auth_client_dir = strdup (MUNGE_AUTH_CLIENT_DIR))) {
         log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
-            "Cannot dup auth-client-dir default string");
+            "Failed to copy auth-client-dir default string");
     }
 #endif /* AUTH_METHOD_RECVFD_MKFIFO || AUTH_METHOD_RECVFD_MKNOD */
 
-    conf->auth_rnd_bytes = MUNGE_AUTH_RND_BYTES;
-
     return (conf);
 }
 
@@ -173,12 +195,17 @@ void
 destroy_conf (conf_t conf)
 {
     assert (conf != NULL);
-    assert (conf->ld < 0);              /* munge_sock_destroy already called */
+    assert (conf->ld < 0);              /* sock_destroy() already called */
+    assert (conf->lockfile_fd < 0);
 
     if (conf->config_name) {
         free (conf->config_name);
         conf->config_name = NULL;
     }
+    if (conf->lockfile_name) {
+        free (conf->lockfile_name);
+        conf->lockfile_name = NULL;
+    }
     if (conf->logfile_name) {
         free (conf->logfile_name);
         conf->logfile_name = NULL;
@@ -262,25 +289,27 @@ parse_cmdline (conf_t conf, int argc, char **argv)
             case 'F':
                 conf->got_foreground = 1;
                 break;
+            case 'M':
+                conf->got_mlockall = 1;
+                break;
             case 'S':
                 if (conf->socket_name)
                     free (conf->socket_name);
                 if (!(conf->socket_name = strdup (optarg)))
                     log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
-                        "Cannot dup socket name string");
+                        "Failed to copy socket name string");
                 break;
-            case 'A':
+            case OPT_ADVICE:
                 printf ("Don't Panic!\n");
                 exit (42);
-            /* Begin deprecated cmdline opts */
-            case '0':
+            case OPT_KEY_FILE:
                 if (conf->key_name)
                     free (conf->key_name);
                 if (!(conf->key_name = strdup (optarg)))
                     log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
-                        "Cannot dup key-file name string");
+                        "Failed to copy key-file name string");
                 break;
-            case '1':
+            case OPT_NUM_THREADS:
                 errno = 0;
                 l = strtol (optarg, &p, 10);
                 if (((errno == ERANGE) && ((l == LONG_MIN) || (l == LONG_MAX)))
@@ -292,22 +321,25 @@ parse_cmdline (conf_t conf, int argc, char **argv)
                 conf->nthreads = l;
                 break;
 #if defined(AUTH_METHOD_RECVFD_MKFIFO) || defined(AUTH_METHOD_RECVFD_MKNOD)
-            case '2':
+            case OPT_AUTH_SERVER:
                 if (conf->auth_server_dir)
                     free (conf->auth_server_dir);
                 if (!(conf->auth_server_dir = strdup (optarg)))
                     log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
-                        "Cannot dup auth-server-dir cmdline string");
+                        "Failed to copy auth-server-dir cmdline string");
                 break;
-            case '3':
+            case OPT_AUTH_CLIENT:
                 if (conf->auth_client_dir)
                     free (conf->auth_client_dir);
                 if (!(conf->auth_client_dir = strdup (optarg)))
                     log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
-                        "Cannot dup auth-client-dir cmdline string");
+                        "Failed to copy auth-client-dir cmdline string");
                 break;
 #endif /* AUTH_METHOD_RECVFD_MKFIFO || AUTH_METHOD_RECVFD_MKNOD */
-            case '4':
+            case OPT_BENCHMARK:
+                conf->got_benchmark = 1;
+                break;
+            case OPT_GROUP_CHECK:
                 errno = 0;
                 l = strtol (optarg, &p, 10);
                 if (((errno == ERANGE) && ((l == LONG_MIN) || (l == LONG_MAX)))
@@ -317,7 +349,7 @@ parse_cmdline (conf_t conf, int argc, char **argv)
                 }
                 conf->got_group_stat = !! l;
                 break;
-            case '5':
+            case OPT_GROUP_UPDATE:
                 errno = 0;
                 l = strtol (optarg, &p, 10);
                 if (((errno == ERANGE) && ((l == LONG_MIN) || (l == LONG_MAX)))
@@ -328,7 +360,9 @@ parse_cmdline (conf_t conf, int argc, char **argv)
                 }
                 conf->gids_update_secs = l;
                 break;
-            /* End deprecated cmdline opts */
+            case OPT_SYSLOG:
+                conf->got_syslog = 1;
+                break;
             case '?':
                 if (optopt > 0) {
                     log_err (EMUNGE_SNAFU, LOG_ERR,
@@ -340,7 +374,7 @@ parse_cmdline (conf_t conf, int argc, char **argv)
                 }
                 else {
                     log_err (EMUNGE_SNAFU, LOG_ERR,
-                        "Unable to process command-line");
+                        "Failed to process command-line");
                 }
                 break;
             case ':':
@@ -356,7 +390,7 @@ parse_cmdline (conf_t conf, int argc, char **argv)
                 }
                 else {
                     log_err (EMUNGE_SNAFU, LOG_ERR,
-                        "Unable to process command-line");
+                        "Failed to process command-line");
                 }
                 break;
             default:
@@ -404,16 +438,17 @@ display_help (char *prog)
     printf ("\n");
 
     printf ("  %*s %s\n", w, "-f, --force",
-            "Force process to run if possible");
+            "Force daemon to run if possible");
 
     printf ("  %*s %s\n", w, "-F, --foreground",
-            "Run process in the foreground (do not fork)");
+            "Run daemon in the foreground (do not fork)");
+
+    printf ("  %*s %s\n", w, "-M, --mlockall",
+            "Lock all pages in memory");
 
     printf ("  %*s %s [%s]\n", w, "-S, --socket=PATH",
             "Specify local socket", MUNGE_SOCKET_NAME);
 
-    /* Begin deprecated cmdline opts
-     */
     printf ("\n");
 
 #if defined(AUTH_METHOD_RECVFD_MKFIFO) || defined(AUTH_METHOD_RECVFD_MKNOD)
@@ -424,6 +459,9 @@ display_help (char *prog)
             "Specify auth-client directory", MUNGE_AUTH_CLIENT_DIR);
 #endif /* AUTH_METHOD_RECVFD_MKFIFO || AUTH_METHOD_RECVFD_MKNOD */
 
+    printf ("  %*s %s\n", w, "--benchmark",
+            "Disable timers to reduce noise while benchmarking");
+
     printf ("  %*s Specify whether to check \"%s\" mtime [%d]\n",
             w, "--group-check-mtime=BOOL", GIDS_GROUP_FILE,
             MUNGE_GROUP_STAT_FLAG);
@@ -437,8 +475,10 @@ display_help (char *prog)
 
     printf ("  %*s %s [%d]\n", w, "--num-threads=INT",
             "Specify number of threads to spawn", MUNGE_THREADS);
-    /* End deprecated cmdline opts
-     */
+
+    printf ("  %*s %s\n", w, "--syslog",
+            "Redirect log messages to syslog");
+
     printf ("\n");
     return;
 }
@@ -462,25 +502,25 @@ create_subkeys (conf_t conf)
      */
     if ((conf->dek_key_len = md_size (MUNGE_MAC_SHA1)) <= 0) {
         log_err (EMUNGE_NO_MEMORY, LOG_ERR,
-            "Unable to determine dek key length");
+            "Failed to determine DEK key length");
     }
     if (!(conf->dek_key = malloc (conf->dek_key_len))) {
         log_err (EMUNGE_NO_MEMORY, LOG_ERR,
-            "Unable to allocate %d bytes for cipher subkey",
+            "Failed to allocate %d bytes for cipher subkey",
             conf->dek_key_len);
     }
     if ((conf->mac_key_len = md_size (MUNGE_MAC_SHA1)) <= 0) {
         log_err (EMUNGE_NO_MEMORY, LOG_ERR,
-            "Unable to determine mac key length");
+            "Failed to determine MAC key length");
     }
     if (!(conf->mac_key = malloc (conf->mac_key_len))) {
         log_err (EMUNGE_NO_MEMORY, LOG_ERR,
-            "Unable to allocate %d bytes for mac subkey",
+            "Failed to allocate %d bytes for MAC subkey",
             conf->mac_key_len);
     }
     if (md_init (&dek_ctx, MUNGE_MAC_SHA1) < 0) {
         log_err (EMUNGE_SNAFU, LOG_ERR,
-            "Unable to compute subkeys: Cannot init md ctx");
+            "Failed to compute subkeys: Cannot init md ctx");
     }
     /*  Compute keyfile's message digest.
      */
@@ -496,14 +536,15 @@ create_subkeys (conf_t conf)
             continue;
         if (n < 0)
             log_errno (EMUNGE_SNAFU, LOG_ERR,
-                "Unable to read keyfile \"%s\"", conf->key_name);
+                "Failed to read keyfile \"%s\"", conf->key_name);
         if (md_update (&dek_ctx, buf, n) < 0)
-            log_err (EMUNGE_SNAFU, LOG_ERR, "Unable to compute subkeys");
+            log_err (EMUNGE_SNAFU, LOG_ERR,
+                "Failed to compute subkeys: Cannot update md ctx");
         n_total += n;
     }
     if (close (fd) < 0) {
         log_errno (EMUNGE_SNAFU, LOG_ERR,
-            "Unable to close keyfile \"%s\"", conf->key_name);
+            "Failed to close keyfile \"%s\"", conf->key_name);
     }
     if (n_total < MUNGE_MINIMUM_SECRET_KEY_LEN) {
         log_err (EMUNGE_SNAFU, LOG_ERR,
@@ -511,7 +552,7 @@ create_subkeys (conf_t conf)
     }
     if (md_copy (&mac_ctx, &dek_ctx) < 0) {
         log_err (EMUNGE_SNAFU, LOG_ERR,
-            "Unable to compute subkeys: Cannot copy md ctx");
+            "Failed to compute subkeys: Cannot copy md ctx");
     }
     /*  Append "1" to keyfile in order to compute cipher subkey.
      */
@@ -519,7 +560,7 @@ create_subkeys (conf_t conf)
     if ( (md_update (&dek_ctx, "1", 1) < 0)
       || (md_final (&dek_ctx, conf->dek_key, &n) < 0)
       || (md_cleanup (&dek_ctx) < 0) ) {
-        log_err (EMUNGE_SNAFU, LOG_ERR, "Unable to compute cipher subkey");
+        log_err (EMUNGE_SNAFU, LOG_ERR, "Failed to compute cipher subkey");
     }
     assert (n <= conf->dek_key_len);
 
@@ -529,7 +570,7 @@ create_subkeys (conf_t conf)
     if ( (md_update (&mac_ctx, "2", 1) < 0)
       || (md_final (&mac_ctx, conf->mac_key, &n) < 0)
       || (md_cleanup (&mac_ctx) < 0) ) {
-        log_err (EMUNGE_SNAFU, LOG_ERR, "Unable to compute mac subkey");
+        log_err (EMUNGE_SNAFU, LOG_ERR, "Failed to compute MAC subkey");
     }
     assert (n <= conf->mac_key_len);
 
@@ -541,34 +582,45 @@ void
 lookup_ip_addr (conf_t conf)
 {
     char hostname [MAXHOSTNAMELEN];
-    char ip_buf [INET_ADDRSTRLEN];
+    char ip_addr_buf [INET_ADDRSTRLEN];
     struct hostent *hptr;
 
     if (gethostname (hostname, sizeof (hostname)) < 0) {
-        log_errno (EMUNGE_SNAFU, LOG_ERR, "Unable to determine hostname");
+        log_errno (EMUNGE_SNAFU, LOG_ERR, "Failed to determine hostname");
     }
-    /*  The man page doesn't say what happens if the buffer is overrun,
-     *    so guarantee buffer NUL-termination just in case.
-     */
     hostname [sizeof (hostname) - 1] = '\0';
-    /*
-     *  The gethostbyname() call is not reentrant, but that's ok because:
+
+    /*  The origin IP address is embedded within the credential metadata,
+     *    but is informational and not required for successful authentication.
+     *    The in_addr is zeroed here so that if name resolution fails, the IPv4
+     *    address will be set to 0.0.0.0.
+     */
+    memset (&conf->addr, 0, sizeof (conf->addr));
+
+    /*  The gethostbyname() call is not reentrant, but that's ok because:
      *    1. there is only one thread active at this point, and
      *    2. this is the only call to gethostbyname().
-     *
      *  Note that gethostbyname() DOES NOT set errno.
      */
     if (!(hptr = gethostbyname (hostname))) {
-        log_err (EMUNGE_SNAFU, LOG_ERR,
-            "Unable to resolve host \"%s\"", hostname);
+        log_msg (LOG_WARNING, "Failed to resolve host \"%s\"", hostname);
+    }
+    else if (sizeof (conf->addr) != hptr->h_length) {
+        log_msg (LOG_WARNING,
+                "Failed to resolve host \"%s\": not an IPv4 address (len=%d)",
+                hostname, hptr->h_length);
+    }
+    else {
+        memcpy (&conf->addr, hptr->h_addr_list[0], sizeof (conf->addr));
     }
-    assert (sizeof (conf->addr) == hptr->h_length);
-    memcpy (&conf->addr, hptr->h_addr_list[0], sizeof (conf->addr));
 
-    if (!inet_ntop (AF_INET, &conf->addr, ip_buf, sizeof (ip_buf))) {
-        log_errno (EMUNGE_SNAFU, LOG_ERR, "Unable to determine ip address");
+    if (!inet_ntop (AF_INET, &conf->addr, ip_addr_buf, sizeof (ip_addr_buf))) {
+        log_errno (EMUNGE_SNAFU, LOG_ERR,
+                "Failed to convert IP address for \"%s\"", hostname);
     }
-    log_msg (LOG_NOTICE, "Running on \"%s\" (%s)", hptr->h_name, ip_buf);
+
+    log_msg (LOG_NOTICE, "Running on \"%s\" (%s)",
+            (hptr && hptr->h_name) ? hptr->h_name : hostname, ip_addr_buf);
     return;
 }
 
@@ -598,7 +650,7 @@ _conf_open_keyfile (const char *keyfile, int got_force)
 
     if (stat (keyfile, &st) < 0) {
         log_errno (EMUNGE_SNAFU, LOG_ERR,
-            "Cannot check keyfile \"%s\"", keyfile);
+            "Failed to check keyfile \"%s\"", keyfile);
     }
     if (!S_ISREG (st.st_mode) || got_symlink) {
         if (!got_force)
@@ -613,11 +665,11 @@ _conf_open_keyfile (const char *keyfile, int got_force)
     if (st.st_uid != geteuid ()) {
         if (!got_force)
             log_err (EMUNGE_SNAFU, LOG_ERR,
-                "Keyfile is insecure: \"%s\" should be owned by uid=%u",
+                "Keyfile is insecure: \"%s\" should be owned by UID %u",
                 keyfile, (unsigned) geteuid ());
         else
             log_msg (LOG_WARNING,
-                "Keyfile is insecure: \"%s\" should be owned by uid=%u",
+                "Keyfile is insecure: \"%s\" should be owned by UID %u",
                 keyfile, (unsigned) geteuid ());
     }
     if (st.st_mode & (S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH)) {
@@ -634,12 +686,12 @@ _conf_open_keyfile (const char *keyfile, int got_force)
      */
     if (path_dirname (keyfile, keydir, sizeof (keydir)) < 0) {
         log_err (EMUNGE_SNAFU, LOG_ERR,
-            "Cannot determine dirname of keyfile \"%s\"", keyfile);
+            "Failed to determine dirname of keyfile \"%s\"", keyfile);
     }
     n = path_is_secure (keydir, ebuf, sizeof (ebuf));
     if (n < 0) {
         log_err (EMUNGE_SNAFU, LOG_ERR,
-            "Cannot check keyfile dir \"%s\": %s", keydir, ebuf);
+            "Failed to check keyfile dir \"%s\": %s", keydir, ebuf);
     }
     else if ((n == 0) && (!got_force)) {
         log_err (EMUNGE_SNAFU, LOG_ERR, "Keyfile is insecure: %s", ebuf);
@@ -651,7 +703,7 @@ _conf_open_keyfile (const char *keyfile, int got_force)
      */
     if ((fd = open (keyfile, O_RDONLY)) < 0) {
         log_errno (EMUNGE_SNAFU, LOG_ERR,
-            "Unable to open keyfile \"%s\"", keyfile);
+            "Failed to open keyfile \"%s\"", keyfile);
     }
     return (fd);
 }
