@@ -1,13 +1,11 @@
 /*****************************************************************************
- *  $Id: dec.c 890 2011-01-20 01:54:21Z chris.m.dunlap $
- *****************************************************************************
  *  Written by Chris Dunlap <cdunlap@llnl.gov>.
- *  Copyright (C) 2007-2011 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2013 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  UCRL-CODE-155910.
  *
  *  This file is part of the MUNGE Uid 'N' Gid Emporium (MUNGE).
- *  For details, see <http://munge.googlecode.com/>.
+ *  For details, see <https://munge.googlecode.com/>.
  *
  *  MUNGE is free software: you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
@@ -45,6 +43,7 @@
 #include "cipher.h"
 #include "conf.h"
 #include "cred.h"
+#include "crypto.h"
 #include "dec.h"
 #include "gids.h"
 #include "log.h"
@@ -181,7 +180,7 @@ dec_timestamp (munge_cred_t c)
      */
     if (time (&now) == ((time_t) -1)) {
         return (m_msg_set_err (m, EMUNGE_SNAFU,
-            strdup ("Unable to query current time")));
+            strdup ("Failed to query current time")));
     }
     m->time0 = 0;
     m->time1 = now;                     /* potential 64b value for 32b var */
@@ -205,7 +204,7 @@ dec_authenticate (munge_cred_t c)
      */
     if (auth_recv (m, p_uid, p_gid) != EMUNGE_SUCCESS) {
         return (m_msg_set_err (m, EMUNGE_SNAFU,
-            strdup ("Unable to determine identity of client")));
+            strdup ("Failed to determine client identity")));
     }
     return (0);
 }
@@ -225,7 +224,7 @@ dec_check_retry (munge_cred_t c)
     }
     if (m->retry > MUNGE_SOCKET_RETRY_ATTEMPTS) {
         return (m_msg_set_err (m, EMUNGE_SOCKET,
-            strdupf ("Exceeded maximum transaction retry attempts")));
+            strdupf ("Exceeded maximum number of decode attempts")));
     }
     return (0);
 }
@@ -267,7 +266,7 @@ dec_unarmor (munge_cred_t c)
     if (prefix_len > 0) {
         if (strncmp ((char *) base64_ptr, MUNGE_CRED_PREFIX, prefix_len)) {
             return (m_msg_set_err (m, EMUNGE_BAD_CRED,
-                strdup ("Unable to match armor prefix")));
+                strdup ("Failed to match armor prefix")));
         }
         base64_ptr += prefix_len;
         base64_len -= prefix_len;
@@ -293,7 +292,7 @@ dec_unarmor (munge_cred_t c)
         }
         if (base64_tmp < base64_ptr) {
             return (m_msg_set_err (m, EMUNGE_BAD_CRED,
-                strdup ("Unable to match armor suffix")));
+                strdup ("Failed to match armor suffix")));
         }
         base64_len = base64_tmp - base64_ptr;
     }
@@ -307,7 +306,7 @@ dec_unarmor (munge_cred_t c)
      */
     if (base64_decode_block (c->outer_mem, &n, base64_ptr, base64_len) < 0) {
         return (m_msg_set_err (m, EMUNGE_BAD_CRED,
-            strdup ("Unable to base64-decode credential")));
+            strdup ("Failed to base64-decode credential")));
     }
     assert (n < c->outer_mem_len);
 
@@ -370,7 +369,7 @@ dec_unpack_outer (munge_cred_t c)
     c->version = *p;
     if (c->version != MUNGE_CRED_VERSION) {
         return (m_msg_set_err (m, EMUNGE_BAD_VERSION,
-            strdupf ("Unsupported credential version %d", c->version)));
+            strdupf ("Invalid credential version %d", c->version)));
     }
     p += n;
     len -= n;
@@ -381,7 +380,7 @@ dec_unpack_outer (munge_cred_t c)
     assert (n == 1);
     if (n > len) {
         return (m_msg_set_err (m, EMUNGE_BAD_CRED,
-            strdup ("Truncated credential cipher type")));
+            strdup ("Truncated cipher type")));
     }
     m->cipher = *p;
     if (m->cipher == MUNGE_CIPHER_NONE) {
@@ -395,7 +394,7 @@ dec_unpack_outer (munge_cred_t c)
         c->iv_len = cipher_iv_size (m->cipher);
         if (c->iv_len < 0) {
             return (m_msg_set_err (m, EMUNGE_SNAFU,
-                strdupf ("Unable to determine iv length for cipher type %d",
+                strdupf ("Failed to determine IV length for cipher type %d",
                 m->cipher)));
         }
         assert (c->iv_len <= sizeof (c->iv));
@@ -409,17 +408,17 @@ dec_unpack_outer (munge_cred_t c)
     assert (n == 1);
     if (n > len) {
         return (m_msg_set_err (m, EMUNGE_BAD_CRED,
-            strdup ("Truncated credential mac type")));
+            strdup ("Truncated MAC type")));
     }
     m->mac = *p;
     if (mac_map_enum (m->mac, NULL) < 0) {
         return (m_msg_set_err (m, EMUNGE_BAD_MAC,
-            strdupf ("Invalid mac type %d", m->mac)));
+            strdupf ("Invalid MAC type %d", m->mac)));
     }
     c->mac_len = mac_size (m->mac);
     if (c->mac_len <= 0) {
         return (m_msg_set_err (m, EMUNGE_SNAFU,
-            strdupf ("Unable to determine digest length for mac type %d",
+            strdupf ("Failed to determine digest length for MAC type %d",
             m->mac)));
     }
     assert (c->mac_len <= sizeof (c->mac));
@@ -432,7 +431,7 @@ dec_unpack_outer (munge_cred_t c)
      */
     if (mac_size (m->mac) < cipher_key_size (m->cipher)) {
         return (m_msg_set_err (m, EMUNGE_BAD_MAC,
-            strdupf ("Invalid mac type %d with cipher type %d",
+            strdupf ("Invalid MAC type %d with cipher type %d",
             m->mac, m->cipher)));
     }
     /*
@@ -442,7 +441,7 @@ dec_unpack_outer (munge_cred_t c)
     assert (n == 1);
     if (n > len) {
         return (m_msg_set_err (m, EMUNGE_BAD_CRED,
-            strdup ("Truncated credential compression type")));
+            strdup ("Truncated compression type")));
     }
     m->zip = *p;
     if (m->zip == MUNGE_ZIP_NONE) {
@@ -463,7 +462,7 @@ dec_unpack_outer (munge_cred_t c)
     assert (n == 1);
     if (n > len) {
         return (m_msg_set_err (m, EMUNGE_BAD_CRED,
-            strdup ("Truncated credential realm length")));
+            strdup ("Truncated security realm length")));
     }
     m->realm_len = *p;
     p += n;
@@ -475,7 +474,7 @@ dec_unpack_outer (munge_cred_t c)
     if (m->realm_len > 0) {
         if (m->realm_len > len) {
             return (m_msg_set_err (m, EMUNGE_BAD_CRED,
-                strdup ("Truncated credential realm string")));
+                strdup ("Truncated security realm string")));
         }
         c->realm_mem_len = m->realm_len + 1;
         /*
@@ -502,7 +501,7 @@ dec_unpack_outer (munge_cred_t c)
     if (c->iv_len > 0) {
         if (c->iv_len > len) {
             return (m_msg_set_err (m, EMUNGE_BAD_CRED,
-                strdup ("Truncated credential iv")));
+                strdup ("Truncated cipher IV")));
         }
         assert (c->iv_len <= sizeof (c->iv));
         memcpy (c->iv, p, c->iv_len);
@@ -565,7 +564,7 @@ dec_decrypt (munge_cred_t c)
     c->dek_len = mac_size (m->mac);
     if (c->dek_len <= 0) {
         return (m_msg_set_err (m, EMUNGE_SNAFU,
-            strdupf ("Unable to determine dek key length for mac type %d",
+            strdupf ("Failed to determine DEK key length for MAC type %d",
                 m->mac)));
     }
     assert (c->dek_len <= sizeof (c->dek));
@@ -574,7 +573,7 @@ dec_decrypt (munge_cred_t c)
     if (mac_block (m->mac, conf->dek_key, conf->dek_key_len,
             c->dek, &n, c->mac, c->mac_len) < 0) {
         return (m_msg_set_err (m, EMUNGE_SNAFU,
-            strdup ("Unable to compute dek")));
+            strdup ("Failed to compute DEK")));
     }
     assert (n <= c->dek_len);
     assert (n >= cipher_key_size (m->cipher));
@@ -585,7 +584,7 @@ dec_decrypt (munge_cred_t c)
     n = cipher_block_size (m->cipher);
     if (n <= 0) {
         return (m_msg_set_err (m, EMUNGE_SNAFU,
-            strdupf ("Unable to determine block size for cipher type %d",
+            strdupf ("Failed to determine block size for cipher type %d",
                 m->cipher)));
     }
     buf_len = c->inner_len + n;
@@ -633,7 +632,7 @@ err:
     memset (buf, 0, buf_len);
     free (buf);
     return (m_msg_set_err (m, EMUNGE_SNAFU,
-        strdup ("Unable to decrypt credential")));
+        strdup ("Failed to decrypt credential")));
 }
 
 
@@ -670,7 +669,7 @@ dec_validate_mac (munge_cred_t c)
 
     /*  Validate new computed MAC against old received MAC.
      */
-    if ((n != c->mac_len) || (memcmp (mac, c->mac, c->mac_len) != 0)) {
+    if ((n != c->mac_len) || (crypto_memcmp (mac, c->mac, c->mac_len) != 0)) {
         return (m_msg_set_err (m, EMUNGE_CRED_INVALID, NULL));
     }
     /*  Ensure an invalid cred error from before is caught
@@ -685,7 +684,7 @@ err_cleanup:
     mac_cleanup (&x);
 err:
     return (m_msg_set_err (m, EMUNGE_SNAFU,
-        strdup ("Unable to mac credential")));
+        strdup ("Failed to MAC credential")));
 }
 
 
@@ -742,7 +741,7 @@ dec_decompress (munge_cred_t c)
 
 err:
     return (m_msg_set_err (m, EMUNGE_SNAFU,
-        strdup ("Unable to decompress credential")));
+        strdup ("Failed to decompress credential")));
 }
 
 
@@ -782,7 +781,7 @@ dec_unpack_inner (munge_cred_t c)
     assert (c->salt_len <= sizeof (c->salt));
     if (c->salt_len > len) {
         return (m_msg_set_err (m, EMUNGE_BAD_CRED,
-            strdup ("Truncated credential salt")));
+            strdup ("Truncated salt")));
     }
     memcpy (c->salt, p, c->salt_len);
     if (m->cipher != MUNGE_CIPHER_NONE) {
@@ -797,7 +796,7 @@ dec_unpack_inner (munge_cred_t c)
     assert (n == 1);
     if (n > len) {
         return (m_msg_set_err (m, EMUNGE_BAD_CRED,
-            strdup ("Truncated credential origin ip addr length")));
+            strdup ("Truncated origin IP addr length")));
     }
     m->addr_len = *p;                   /* a single byte is always aligned */
     p += n;
@@ -807,11 +806,11 @@ dec_unpack_inner (munge_cred_t c)
      */
     if (m->addr_len != sizeof (m->addr)) {
         return (m_msg_set_err (m, EMUNGE_BAD_CRED,
-            strdup ("Invalid credential origin ip addr length")));
+            strdup ("Invalid origin IP addr length")));
     }
     if (m->addr_len > len) {
         return (m_msg_set_err (m, EMUNGE_BAD_CRED,
-            strdup ("Truncated credential origin ip addr")));
+            strdup ("Truncated origin IP addr")));
     }
     memcpy (&m->addr, p, m->addr_len);
     p += m->addr_len;
@@ -823,7 +822,7 @@ dec_unpack_inner (munge_cred_t c)
     assert (n == 4);
     if (n > len) {
         return (m_msg_set_err (m, EMUNGE_BAD_CRED,
-            strdup ("Truncated credential encode time")));
+            strdup ("Truncated encode time")));
     }
     memcpy (&u, p, n);                  /* ensure proper byte-alignment */
     m->time0 = ntohl (u);
@@ -836,7 +835,7 @@ dec_unpack_inner (munge_cred_t c)
     assert (n == 4);
     if (n > len) {
         return (m_msg_set_err (m, EMUNGE_BAD_CRED,
-            strdup ("Truncated credential time-to-live")));
+            strdup ("Truncated time-to-live")));
     }
     memcpy (&u, p, n);                  /* ensure proper byte-alignment */
     m->ttl = ntohl (u);
@@ -849,7 +848,7 @@ dec_unpack_inner (munge_cred_t c)
     assert (n == 4);
     if (n > len) {
         return (m_msg_set_err (m, EMUNGE_BAD_CRED,
-            strdup ("Truncated credential uid")));
+            strdup ("Truncated UID")));
     }
     memcpy (&u, p, n);                  /* ensure proper byte-alignment */
     m->cred_uid = ntohl (u);
@@ -862,7 +861,7 @@ dec_unpack_inner (munge_cred_t c)
     assert (n == 4);
     if (n > len) {
         return (m_msg_set_err (m, EMUNGE_BAD_CRED,
-            strdup ("Truncated credential gid")));
+            strdup ("Truncated GID")));
     }
     memcpy (&u, p, n);                  /* ensure proper byte-alignment */
     m->cred_gid = ntohl (u);
@@ -875,7 +874,7 @@ dec_unpack_inner (munge_cred_t c)
     assert (n == 4);
     if (n > len) {
         return (m_msg_set_err (m, EMUNGE_BAD_CRED,
-            strdup ("Truncated uid restriction")));
+            strdup ("Truncated UID restriction")));
     }
     memcpy (&u, p, n);                  /* ensure proper byte-alignment */
     m->auth_uid = ntohl (u);
@@ -888,7 +887,7 @@ dec_unpack_inner (munge_cred_t c)
     assert (n == 4);
     if (n > len) {
         return (m_msg_set_err (m, EMUNGE_BAD_CRED,
-            strdup ("Truncated gid restriction")));
+            strdup ("Truncated GID restriction")));
     }
     memcpy (&u, p, n);                  /* ensure proper byte-alignment */
     m->auth_gid = ntohl (u);
@@ -901,7 +900,7 @@ dec_unpack_inner (munge_cred_t c)
     assert (n == 4);
     if (n > len) {
         return (m_msg_set_err (m, EMUNGE_BAD_CRED,
-            strdup ("Truncated credential data length")));
+            strdup ("Truncated data length")));
     }
     memcpy (&u, p, n);                  /* ensure proper byte-alignment */
     m->data_len = ntohl (u);
@@ -915,7 +914,7 @@ dec_unpack_inner (munge_cred_t c)
     if (m->data_len > 0) {
         if (m->data_len > len) {
             return (m_msg_set_err (m, EMUNGE_BAD_CRED,
-                strdup ("Truncated credential data")));
+                strdup ("Truncated data")));
         }
         m->data = p;                    /* data resides in (inner|outer)_mem */
         p += m->data_len;
@@ -955,7 +954,7 @@ dec_validate_auth (munge_cred_t c)
 
 unauthorized:
     return (m_msg_set_err (m, EMUNGE_CRED_UNAUTHORIZED,
-        strdupf ("Unauthorized credential for client uid=%d gid=%d",
+        strdupf ("Unauthorized credential for client UID=%d GID=%d",
             m->client_uid, m->client_gid)));
 }
 
@@ -1014,7 +1013,7 @@ dec_validate_replay (munge_cred_t c)
                 && (m->retry > 0)
                 && (m->retry <= MUNGE_SOCKET_RETRY_ATTEMPTS)) {
             log_msg (LOG_NOTICE,
-                "Allowed credential replay for client uid=%d gid=%d",
+                "Allowed credential replay for client UID=%d GID=%d",
                 m->client_uid, m->client_gid);
             return (0);
         }
