@@ -1,13 +1,11 @@
 /*****************************************************************************
- *  $Id: gids.c 913 2011-02-09 21:15:12Z chris.m.dunlap $
- *****************************************************************************
  *  Written by Chris Dunlap <cdunlap@llnl.gov>.
- *  Copyright (C) 2007-2011 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2013 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  UCRL-CODE-155910.
  *
  *  This file is part of the MUNGE Uid 'N' Gid Emporium (MUNGE).
- *  For details, see <http://munge.googlecode.com/>.
+ *  For details, see <https://munge.googlecode.com/>.
  *
  *  MUNGE is free software: you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
@@ -38,11 +36,13 @@
 #include <assert.h>
 #include <errno.h>
 #include <pthread.h>
+#include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <munge.h>
+#include "conf.h"
 #include "gids.h"
 #include "hash.h"
 #include "log.h"
@@ -74,7 +74,7 @@
  *****************************************************************************/
 
 #ifndef _GIDS_DEBUG
-#define _GIDS_DEBUG 0
+#define _GIDS_DEBUG     0
 #endif /* !_GIDS_DEBUG */
 
 
@@ -85,7 +85,7 @@
 struct gids {
     pthread_mutex_t     mutex;          /* mutex for accessing struct        */
     hash_t              hash;           /* hash of GIDs mappings             */
-    int                 timer;          /* timer ID for next GIDs map update */
+    long                timer;          /* timer ID for next GIDs map update */
     int                 interval;       /* seconds between GIDs map updates  */
     int                 do_group_stat;  /* true if updates stat group file   */
     time_t              t_last_update;  /* time of last good GIDs map update */
@@ -118,20 +118,20 @@ typedef struct gids_uid  * gids_uid_t;
 static void         _gids_update (gids_t gids);
 static hash_t       _gids_hash_create (void);
 static int          _gids_user_to_uid (hash_t uid_hash,
-                        char *user, uid_t *uid_p, char *buf, size_t buflen);
+                        char *user, uid_t *uidp, xpwbuf_p pwbufp);
 static int          _gids_hash_add (hash_t hash, uid_t uid, gid_t gid);
 static gids_gid_t   _gids_head_alloc (uid_t uid);
 static void         _gids_head_del (gids_gid_t g);
 static gids_node_t  _gids_node_alloc (gid_t gid);
-static int          _gids_node_cmp (uid_t *uid1_p, uid_t *uid2_p);
-static unsigned int _gids_node_key (uid_t *uid_p);
+static int          _gids_node_cmp (uid_t *uid1p, uid_t *uid2p);
+static unsigned int _gids_node_key (uid_t *uidp);
 static gids_uid_t   _gids_uid_alloc (char *user, uid_t uid);
 static int          _gids_uid_cmp (char *user1, char *user2);
 static void         _gids_uid_del (gids_uid_t u);
 
 #if _GIDS_DEBUG
 static void _gids_dump_gid_hash (hash_t gid_hash);
-static void _gids_dump_gid_node (gids_gid_t g, uid_t *uid_p, void *null);
+static void _gids_dump_gid_node (gids_gid_t g, uid_t *uidp, void *null);
 static void _gids_dump_uid_hash (hash_t uid_hash);
 static void _gids_dump_uid_node (gids_uid_t u, char *user, void *null);
 #endif /* _GIDS_DEBUG */
@@ -146,18 +146,16 @@ gids_create (int interval, int do_group_stat)
 {
     gids_t gids;
 
-    /*  If the GIDs update interval is negative, disable the GIDs mapping.
-     */
-    if (interval < 0) {
+    if ((interval < 0) || (conf->got_benchmark)) {
         log_msg (LOG_INFO, "Disabled supplementary group mapping");
         return (NULL);
     }
     if (!(gids = malloc (sizeof (*gids)))) {
         log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
-            "Unable to allocate gids struct");
+            "Failed to allocate gids struct");
     }
     if ((errno = pthread_mutex_init (&gids->mutex, NULL)) != 0) {
-        log_errno (EMUNGE_SNAFU, LOG_ERR, "Unable to init gids mutex");
+        log_errno (EMUNGE_SNAFU, LOG_ERR, "Failed to init gids mutex");
     }
     gids->hash = NULL;
     gids->timer = 0;
@@ -190,7 +188,7 @@ gids_destroy (gids_t gids)
         return;
     }
     if ((errno = pthread_mutex_lock (&gids->mutex)) != 0) {
-        log_errno (EMUNGE_SNAFU, LOG_ERR, "Unable to lock gids mutex");
+        log_errno (EMUNGE_SNAFU, LOG_ERR, "Failed to lock gids mutex");
     }
     if (gids->timer > 0) {
         timer_cancel (gids->timer);
@@ -200,12 +198,12 @@ gids_destroy (gids_t gids)
     gids->hash = NULL;
 
     if ((errno = pthread_mutex_unlock (&gids->mutex)) != 0) {
-        log_errno (EMUNGE_SNAFU, LOG_ERR, "Unable to unlock gids mutex");
+        log_errno (EMUNGE_SNAFU, LOG_ERR, "Failed to unlock gids mutex");
     }
     hash_destroy (h);
 
     if ((errno = pthread_mutex_destroy (&gids->mutex)) != 0) {
-        log_msg (LOG_ERR, "Unable to destroy gids mutex: %s",
+        log_msg (LOG_ERR, "Failed to destroy gids mutex: %s",
             strerror (errno));
     }
     free (gids);
@@ -220,7 +218,7 @@ gids_update (gids_t gids)
         return;
     }
     if ((errno = pthread_mutex_lock (&gids->mutex)) != 0) {
-        log_errno (EMUNGE_SNAFU, LOG_ERR, "Unable to lock gids mutex");
+        log_errno (EMUNGE_SNAFU, LOG_ERR, "Failed to lock gids mutex");
     }
     /*  Cancel a pending update before scheduling a new one.
      */
@@ -231,7 +229,7 @@ gids_update (gids_t gids)
      */
     gids->timer = timer_set_relative ((callback_f) _gids_update, gids, 0);
     if (gids->timer < 0) {
-        log_errno (EMUNGE_SNAFU, LOG_ERR, "Unable to set gids update timer");
+        log_errno (EMUNGE_SNAFU, LOG_ERR, "Failed to set gids update timer");
     }
     /*  Reset the do_group_stat flag in case it had been disabled on error
      *    (ie, set to -1).
@@ -239,7 +237,7 @@ gids_update (gids_t gids)
     gids->do_group_stat = !! gids->do_group_stat;
 
     if ((errno = pthread_mutex_unlock (&gids->mutex)) != 0) {
-        log_errno (EMUNGE_SNAFU, LOG_ERR, "Unable to unlock gids mutex");
+        log_errno (EMUNGE_SNAFU, LOG_ERR, "Failed to unlock gids mutex");
     }
     return;
 }
@@ -256,7 +254,7 @@ gids_is_member (gids_t gids, uid_t uid, gid_t gid)
         return (0);
     }
     if ((errno = pthread_mutex_lock (&gids->mutex)) != 0) {
-        log_errno (EMUNGE_SNAFU, LOG_ERR, "Unable to lock gids mutex");
+        log_errno (EMUNGE_SNAFU, LOG_ERR, "Failed to lock gids mutex");
     }
     if ((gids->hash) && (g = hash_find (gids->hash, &uid))) {
         assert (g->uid == uid);
@@ -268,7 +266,7 @@ gids_is_member (gids_t gids, uid_t uid, gid_t gid)
         }
     }
     if ((errno = pthread_mutex_unlock (&gids->mutex)) != 0) {
-        log_errno (EMUNGE_SNAFU, LOG_ERR, "Unable to unlock gids mutex");
+        log_errno (EMUNGE_SNAFU, LOG_ERR, "Failed to unlock gids mutex");
     }
     return (is_member);
 }
@@ -292,16 +290,16 @@ _gids_update (gids_t gids)
     assert (gids != NULL);
 
     if ((errno = pthread_mutex_lock (&gids->mutex)) != 0) {
-        log_errno (EMUNGE_SNAFU, LOG_ERR, "Unable to lock gids mutex");
+        log_errno (EMUNGE_SNAFU, LOG_ERR, "Failed to lock gids mutex");
     }
     do_group_stat = gids->do_group_stat;
     t_last_update = gids->t_last_update;
 
     if ((errno = pthread_mutex_unlock (&gids->mutex)) != 0) {
-        log_errno (EMUNGE_SNAFU, LOG_ERR, "Unable to unlock gids mutex");
+        log_errno (EMUNGE_SNAFU, LOG_ERR, "Failed to unlock gids mutex");
     }
     if (time (&t_now) == (time_t) -1) {
-        log_errno (EMUNGE_SNAFU, LOG_ERR, "Unable to query current time");
+        log_errno (EMUNGE_SNAFU, LOG_ERR, "Failed to query current time");
     }
     if (do_group_stat > 0) {
 
@@ -311,7 +309,7 @@ _gids_update (gids_t gids)
          */
         if (stat (GIDS_GROUP_FILE, &st) < 0) {
             do_group_stat = -2;
-            log_msg (LOG_ERR, "Unable to stat \"%s\": %s",
+            log_msg (LOG_ERR, "Failed to stat \"%s\": %s",
                 GIDS_GROUP_FILE, strerror (errno));
         }
         else if (st.st_mtime <= t_last_update) {
@@ -324,7 +322,7 @@ _gids_update (gids_t gids)
         hash = _gids_hash_create ();
     }
     if ((errno = pthread_mutex_lock (&gids->mutex)) != 0) {
-        log_errno (EMUNGE_SNAFU, LOG_ERR, "Unable to lock gids mutex");
+        log_errno (EMUNGE_SNAFU, LOG_ERR, "Failed to lock gids mutex");
     }
     /*  Replace the old GIDs mapping if the update was successful.
      */
@@ -353,11 +351,11 @@ _gids_update (gids_t gids)
                 (callback_f) _gids_update, gids, gids->interval * 1000);
         if (gids->timer < 0) {
             log_errno (EMUNGE_SNAFU, LOG_ERR,
-                "Unable to reset gids update timer");
+                "Failed to reset gids update timer");
         }
     }
     if ((errno = pthread_mutex_unlock (&gids->mutex)) != 0) {
-        log_errno (EMUNGE_SNAFU, LOG_ERR, "Unable to unlock gids mutex");
+        log_errno (EMUNGE_SNAFU, LOG_ERR, "Failed to unlock gids mutex");
     }
     /*  Clean up.
      */
@@ -373,17 +371,19 @@ _gids_hash_create (void)
 {
 /*  Returns a new hash containing the new GIDs mapping, or NULL on error.
  */
+    static size_t   grbuflen = 0;
+    static size_t   pwbuflen = 0;
     hash_t          gid_hash = NULL;
     hash_t          uid_hash = NULL;
     struct timeval  t_start;
     struct timeval  t_stop;
     int             do_group_db_close = 0;
+    int             num_inits = 0;
+    const int       max_inits = 16;
     struct group    gr;
-    char           *gr_buf_ptr = NULL;
-    int             gr_buf_len;
-    char           *pw_buf_ptr = NULL;
-    int             pw_buf_len;
-    char          **user_p;
+    xgrbuf_p        grbufp = NULL;
+    xpwbuf_p        pwbufp = NULL;
+    char          **userp;
     uid_t           uid;
     int             n_users;
     double          n_seconds;
@@ -392,18 +392,18 @@ _gids_hash_create (void)
             (hash_cmp_f) _gids_node_cmp, (hash_del_f) _gids_head_del);
 
     if (!gid_hash) {
-        log_msg (LOG_ERR, "Unable to allocate gids hash -- out of memory");
+        log_msg (LOG_ERR, "Failed to allocate gids hash");
         goto err;
     }
     uid_hash = hash_create (UIDS_HASH_SIZE, (hash_key_f) hash_key_string,
             (hash_cmp_f) _gids_uid_cmp, (hash_del_f) _gids_uid_del);
 
     if (!uid_hash) {
-        log_msg (LOG_ERR, "Unable to allocate uids hash -- out of memory");
+        log_msg (LOG_ERR, "Failed to allocate uids hash");
         goto err;
     }
     if (gettimeofday (&t_start, NULL) < 0) {
-        log_msg (LOG_ERR, "Unable to query current time");
+        log_msg (LOG_ERR, "Failed to query current time");
         goto err;
     }
     /*  Allocate memory for both the xgetgrent() and xgetpwnam() buffers here.
@@ -411,33 +411,40 @@ _gids_hash_create (void)
      *    is used, but allocating it here allows the same buffer to be reused
      *    throughout a given GIDs creation cycle.
      */
-    if (xgetgrent_buf_create (&gr_buf_ptr, &gr_buf_len) < 0) {
-        log_msg (LOG_ERR, "Unable to allocate group entry buffer");
+    if (!(grbufp = xgetgrent_buf_create (grbuflen))) {
+        log_msg (LOG_ERR, "Failed to allocate group entry buffer");
         goto err;
     }
-    if (xgetpwnam_buf_create (&pw_buf_ptr, &pw_buf_len) < 0) {
-        log_msg (LOG_ERR, "Unable to allocate password entry buffer");
+    if (!(pwbufp = xgetpwnam_buf_create (pwbuflen))) {
+        log_msg (LOG_ERR, "Failed to allocate password entry buffer");
         goto err;
     }
-    xgetgrent_init ();
     do_group_db_close = 1;
+restart:
+    xgetgrent_init ();
+    num_inits++;
 
     while (1) {
-        if (xgetgrent (&gr, gr_buf_ptr, gr_buf_len) < 0) {
-            if (errno == ENOENT)
+        if (xgetgrent (&gr, grbufp) < 0) {
+            if (errno == ENOENT) {
                 break;
-            if (errno == EINTR)
+            }
+            if (errno == EINTR) {
                 continue;
-            log_msg (LOG_ERR, "Unable to query group info: %s",
+            }
+            if ((errno == ERANGE) && (num_inits < max_inits)) {
+                hash_reset (gid_hash);
+                goto restart;
+            }
+            log_msg (LOG_ERR, "Failed to query group info: %s",
                     strerror (errno));
             goto err;
         }
         /*  gr_mem is a null-terminated array of pointers to the user names
          *    belonging to the group.
          */
-        for (user_p = gr.gr_mem; user_p && *user_p; user_p++) {
-            int rv = _gids_user_to_uid (uid_hash, *user_p, &uid,
-                    pw_buf_ptr, pw_buf_len);
+        for (userp = gr.gr_mem; userp && *userp; userp++) {
+            int rv = _gids_user_to_uid (uid_hash, *userp, &uid, pwbufp);
             if (rv == 0) {
                 if (_gids_hash_add (gid_hash, uid, gr.gr_gid) < 0) {
                     goto err;
@@ -446,10 +453,18 @@ _gids_hash_create (void)
         }
     }
     xgetgrent_fini ();
-    xgetgrent_buf_destroy (gr_buf_ptr);
+    /*
+     *  Record the final size of the xgetpwnam() and xgetgrent() buffers.
+     *    This allows subsequent scans to start with buffers that will
+     *    generally not need to be realloc()d.
+     */
+    pwbuflen = xgetpwnam_buf_get_len (pwbufp);
+    xgetpwnam_buf_destroy (pwbufp);
+    grbuflen = xgetgrent_buf_get_len (grbufp);
+    xgetgrent_buf_destroy (grbufp);
 
     if (gettimeofday (&t_stop, NULL) < 0) {
-        log_msg (LOG_ERR, "Unable to query current time");
+        log_msg (LOG_ERR, "Failed to query current time");
         goto err;
     }
 
@@ -472,11 +487,11 @@ err:
     if (do_group_db_close) {
         xgetgrent_fini ();
     }
-    if (pw_buf_ptr != NULL) {
-        xgetpwnam_buf_destroy (pw_buf_ptr);
+    if (pwbufp != NULL) {
+        xgetpwnam_buf_destroy (pwbufp);
     }
-    if (gr_buf_ptr != NULL) {
-        xgetgrent_buf_destroy (gr_buf_ptr);
+    if (grbufp != NULL) {
+        xgetgrent_buf_destroy (grbufp);
     }
     if (uid_hash != NULL) {
         hash_destroy (uid_hash);
@@ -489,10 +504,9 @@ err:
 
 
 static int
-_gids_user_to_uid (hash_t uid_hash, char *user, uid_t *uid_p,
-                   char *buf, size_t buflen)
+_gids_user_to_uid (hash_t uid_hash, char *user, uid_t *uidp, xpwbuf_p pwbufp)
 {
-/*  Returns 0 on success, setting [*uid_p] (if non-NULL) to the UID associated
+/*  Returns 0 on success, setting [*uidp] (if non-NULL) to the UID associated
  *    with [user]; o/w, returns -1.
  */
     gids_uid_t     u;
@@ -502,27 +516,27 @@ _gids_user_to_uid (hash_t uid_hash, char *user, uid_t *uid_p,
     if ((u = hash_find (uid_hash, user))) {
         uid = u->uid;
     }
-    else if (xgetpwnam (user, &pw, buf, buflen) == 0) {
+    else if (xgetpwnam (user, &pw, pwbufp) == 0) {
         uid = pw.pw_uid;
         if (!(u = _gids_uid_alloc (user, uid))) {
             log_msg (LOG_WARNING,
-                "Unable to allocate uid node for %s/%d -- out of memory",
+                "Failed to allocate uid node for %s/%d",
                 user, uid);
         }
         else if (!hash_insert (uid_hash, u->user, u)) {
             log_msg (LOG_WARNING,
-                "Unable to insert uid node for %s/%d into hash", user, uid);
+                "Failed to insert uid node for %s/%d into hash", user, uid);
             _gids_uid_del (u);
         }
     }
     else {
         log_msg (LOG_INFO,
-            "Unable to query password file entry for \"%s\"", user);
+            "Failed to query password file entry for \"%s\"", user);
         return (-1);
     }
 
-    if (uid_p != NULL) {
-        *uid_p = uid;
+    if (uidp != NULL) {
+        *uidp = uid;
     }
     return (0);
 }
@@ -537,34 +551,34 @@ _gids_hash_add (hash_t hash, uid_t uid, gid_t gid)
  */
     gids_gid_t   g;
     gids_node_t  node;
-    gids_node_t *node_p;
+    gids_node_t *nodep;
 
     if (!(g = hash_find (hash, &uid))) {
         if (!(g = _gids_head_alloc (uid))) {
-            log_msg (LOG_ERR, "Unable to allocate gids node -- out of memory");
+            log_msg (LOG_ERR, "Failed to allocate gids node");
             return (-1);
         }
         if (!hash_insert (hash, &g->uid, g)) {
-            log_msg (LOG_ERR, "Unable to insert gids node into hash");
+            log_msg (LOG_ERR, "Failed to insert gids node into hash");
             _gids_head_del (g);
             return (-1);
         }
     }
     assert (g->uid == uid);
 
-    node_p = &g->next;
-    while ((*node_p) && ((*node_p)->gid < gid)) {
-        node_p = &(*node_p)->next;
+    nodep = &g->next;
+    while ((*nodep) && ((*nodep)->gid < gid)) {
+        nodep = &(*nodep)->next;
     }
-    if ((*node_p) && ((*node_p)->gid == gid)) {
+    if ((*nodep) && ((*nodep)->gid == gid)) {
         return (0);
     }
     if (!(node = _gids_node_alloc (gid))) {
-        log_msg (LOG_ERR, "Unable to allocate gids node -- out of memory");
+        log_msg (LOG_ERR, "Failed to allocate gids node");
         return (-1);
     }
-    node->next = *node_p;
-    *node_p = node;
+    node->next = *nodep;
+    *nodep = node;
     return (1);
 }
 
@@ -623,20 +637,20 @@ _gids_node_alloc (gid_t gid)
 
 
 static int
-_gids_node_cmp (uid_t *uid1_p, uid_t *uid2_p)
+_gids_node_cmp (uid_t *uid1p, uid_t *uid2p)
 {
-/*  Used by the hash routines to compare hash keys [uid1_p] and [uid2_p].
+/*  Used by the hash routines to compare hash keys [uid1p] and [uid2p].
  */
-    return (!(*uid1_p == *uid2_p));
+    return (!(*uid1p == *uid2p));
 }
 
 
 static unsigned int
-_gids_node_key (uid_t *uid_p)
+_gids_node_key (uid_t *uidp)
 {
-/*  Used by the hash routines to convert [uid_p] into a hash key.
+/*  Used by the hash routines to convert [uidp] into a hash key.
  */
-    return (*uid_p);
+    return (*uidp);
 }
 
 
@@ -707,11 +721,11 @@ _gids_dump_gid_hash (hash_t gid_hash)
 
 
 static void
-_gids_dump_gid_node (gids_gid_t g, uid_t *uid_p, void *null)
+_gids_dump_gid_node (gids_gid_t g, uid_t *uidp, void *null)
 {
     gids_node_t node;
 
-    assert (g->uid == *uid_p);
+    assert (g->uid == *uidp);
 
     printf (" %5d:", g->uid);
     for (node = g->next; node; node = node->next) {
